#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <sys\types.h>

unsigned char soundptr[32000];
char count_values[256];

int dsp_reset, dsp_read_data, dsp_write_data;
int dsp_write_status, dsp_data_avail;

int resetdsp(void)
{
	dsp_reset = 0x226;
	dsp_read_data = 0x22a;
	dsp_write_data = 0x22c;
	dsp_write_status = 0x22c;
	dsp_data_avail = 0x22e;

	outp(dsp_reset, 1);
	delay(10);
	outp(dsp_reset, 0);
	delay(10);

	if ((inp(dsp_data_avail) & 0x80) &&
	    (inp(dsp_read_data) == 0xaa))
	    return(0);

	return(-1);
}

void writedsp(char value)
{
	while ((inp(dsp_write_status) & 0x80) != 0);
	outp(dsp_write_data, value);
}

char readdsp(void)
{
	while((inp(dsp_data_avail) & 0x80) != 0);
	return(inp(dsp_read_data));
}

void writedac(char level)
{
	writedsp(0x10);
	writedsp(level);
}

char readdac(void)
{
	writedsp(0x20);
	return(readdsp());
}

void speakeron(void)
{
	writedsp(0xd1);
}

void speakeroff(void)
{
	writedsp(0xd3);
}

void dmacontinue(void)
{
	writedsp(0xd4);
}

void dmastop(void)
{
	writedsp(0xd0);
}

void playback(char *sound, int size, int freq)
{
	int time_constant;
	unsigned long page, offset;

	offset = ((long)FP_SEG(sound)<<4) + (long)FP_OFF(sound);
	page = (int)((offset>>16)&255);

	time_constant = 256 - 1000000 / freq;
	speakeron();
	writedsp(0x40);
	writedsp(time_constant);

	writedsp(0x14);
	writedsp((size-1)&255);
	writedsp(((size-1)>>8)&255);

	outp(0xa, 5);
	outp(0xc, 0);
	outp(0xb, 0x49);
	outp(0x2, (int)(offset&255));
	outp(0x2, (int)(offset>>8)&255);
	outp(0x3, (size-1)&255);
	outp(0x3, ((size-1)>>8)&255);
	outp(0x83, page);
	outp(0xa, 1);
}

int main(int argc, char *argv[])
{
	long i, fil;
	char checksum;
	short rate, size, filler, sfmt;

	if (argc < 2)
	{
		printf("DIGIPLAY v1.1	- By Michael Muniko\n");
		printf("This will play Prince of Persia 1 digital sounds.\n");
		printf("Syntax: digiplay [sound file]\n");
		exit(0);
	}
	if ((fil = open(argv[1], O_BINARY|O_RDONLY,S_IREAD)) == -1)
	{
		printf("Cannot open %s.\n", argv[1]);
		exit(-1);
	}
	read(fil,&checksum,1);	
	if (checksum != 0x1)
	{
		close(fil);
		printf("What do you think I am? That ain't no POP sound!\n");
		exit(-1);
	}
	read(fil,&rate,2);
	read(fil,&size,2);
	read(fil,&filler,2);
	read(fil,&sfmt,1);

	read(fil,&soundptr[0],size);
	close(fil);
	if (resetdsp() != 0)
	{
		printf("Cannot find Sound Blaster\n");
		exit(-1);
	}
	printf("rate: %d, size: %d\n", rate, size);
	playback(&soundptr[0], size, rate);
	delay(size);
	resetdsp();
	speakeroff();
	return(0);
} 
